/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: LCBicone.cpp                                           */
/*                                                                       */
/*  Revision: V1.0     Date: 22.04.2009    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Generiert aLichtkurven aus ascci-Datei                      */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 22.04.2009
// Updated by: A. James 23.06.2009
// Updated by: Chr. Monstein 04.08.2010 (Phoenix-3 again)
// Updated by: Chr. Monstein 02.09.2010 adaption to Callisto Metsahovi
// Updated by: Chr. Monstein 04.04.2011 more general ...
// Updated by: Chr. Monstein 14.08.2011 width, height = parameter
// Updated by: Chr. Monstein 08.09.2011 dB korrigiert
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include "xyplott.h"
#include "LCBicone.h"
#include "gd.h"
#include "gdfonts.h"
#include "gdfontg.h"
#include "gdfontl.h"
//---------------------------------------------------------------------------
struct Lightcurves
{
    float MEZ;
    float LC1;
    float LC2;
    float LC3;
    float LC4;
    float LC5;
};
Lightcurves LightCurvesBicone[10000]; // ca alle 30sec 1 Messwertepaar
char value[10];
float nameLC1;
float nameLC2;
float nameLC3;
float nameLC4;
float nameLC5;
bool calibrated;
char ylabel[10];

float IMin = 3000.0, IMax = -3000.0;
float IMin_a, IMin_b, IMin_c, IMin_d, IMin_e;
int oldday; //used to create TpreampYYYYMMDD.png for archieve
bool archivPng = false; //used to create TpreampYYYYMMDD.png for archieve
float grad = 25.4; // detector gradient

//---------------------------------------------------------------------------

#pragma package(smart_init)

//---------------------------------------------------------------------------
void __fastcall CLCBicone::Read() // get data out of weather file
{
    IMin = 3000.0;
    IMin_a = IMin, IMin_b = IMin, IMin_c = IMin, IMin_d = IMin, IMin_e = IMin;
    IMax  =-1000.0;
    char buff[180];
    char filename[240];
    float x, a, b, c, d, e;

    TDateTime dtPresent = Now() - (Form1->dayoffset);
    DecodeDate(dtPresent,Year, Month, Day);
    if (Form1->first_pass) oldday = Day;
    if (oldday != Day)
    {
       oldday = Day;
       dtPresent = Now()-1;
       DecodeDate(dtPresent,Year, Month, Day);
       archivPng = true;
    }

    FILE *in;
    //LC20100902_ADU_Metsahovi.txt
    //LC20100902_ADU_SWISS.txt
    sprintf(filename,"%sLC%04u%02u%02u_SFU_%s.txt",Form1->LCpath,Year,Month,Day,Form1->titlecomment);
    if (FileExists(filename))
    {
       calibrated = true;
       sprintf(ylabel,"[SFU]");
    }
    else
    {
       sprintf(filename,"%sLC%04u%02u%02u_ADU_%s.txt",Form1->LCpath,Year,Month,Day,Form1->titlecomment);
       calibrated = false;
       sprintf(ylabel,"[dB]");
    }

    Form1->Label3->Caption = filename;

    if ((in = fopen(filename, "rt")) == NULL)
    {
        //Application->MessageBox("file does not exist", "Error", MB_OK);
    }
    else
    {
      counts = 0;
      while (!feof(in)&& counts < 10000)
      {
          fgets(buff,sizeof(buff),in);

          if (strstr(buff,"Time_UT,"))
          {
             sscanf(buff,"Time_UT,%fMHz,%fMHz,%fMHz,%fMHz,%fMHz",&nameLC1,&nameLC2,&nameLC3,&nameLC4,&nameLC5);
             fgets(buff,sizeof(buff),in);
          }
          else
          {
             sscanf(buff,"%f,%f,%f,%f,%f,%f",&x,&a,&b,&c,&d,&e);
             //           01,02,03,04  01,02,03,04
             LightCurvesBicone[counts].MEZ = x;
             LightCurvesBicone[counts].LC1 = a;
             LightCurvesBicone[counts].LC2 = b;
             LightCurvesBicone[counts].LC3 = c;
             LightCurvesBicone[counts].LC4 = d;
             LightCurvesBicone[counts].LC5 = e;
             if (a > IMax) IMax = a;
             if (a < IMin_a && (a>10)) IMin_a = a;

             if (b > IMax) IMax = b;
             if (b < IMin_b && (b>10)) IMin_b = b;

             if (c > IMax) IMax = c;
             if (c < IMin_c && (c>10)) IMin_c = c;

             if (d > IMax) IMax = d;
             if (d < IMin_d && (d>10)) IMin_d = d;

             if (e > IMax) IMax = e;
             if (e < IMin_e && (e>10)) IMin_e = e;

             if (IMin_a <= IMin) IMin = IMin_a;
             if (IMin_b <= IMin) IMin = IMin_b;
             if (IMin_c <= IMin) IMin = IMin_c;
             if (IMin_d <= IMin) IMin = IMin_d;
             if (IMin_e <= IMin) IMin = IMin_e;

             if (IMax > 50 && calibrated == true) IMax = 50;//nur wenn SFU
             if (IMin <  0 && calibrated == true) IMin =  0;//nur wenn SFU
          }
          counts++;
      }
    }
    fclose(in);
}
//---------------------------------------------------------------------------
void __fastcall CLCBicone::GdImageGenerator(void)
{
    gdImagePtr im_out; // allocate image memory
    int width   = 350; // original 450
    int height  = (int)((float)width*0.618); // Goldener Schnitt
    width  = Form1->p_width;
    height = Form1->p_height;
    int ALeft   = 0;
    int ATop    = 0;
    int ARight  = width;
    int BTop    = 30;
    int BBottom = height-40;
    int BLeft   = 50;
    int BRight  = width-30;
    float YMax, YMin;
    if(!calibrated)
    {
      //YMax = (IMax-IMin)*0.38598116; // =1/255*2500.0/25.4mV/dB
      YMax = (IMax-IMin)/((256.0*Form1->adcdivider)-1)*2500.0/grad; // =ImV/25.4mV/dB
      YMin = 0;
    }
    else
    {
      YMax = IMax;
      YMin = IMin;
    }
    im_out      = gdImageCreate(width, height); // create image
    int white   = gdImageColorAllocate(im_out,255,255,255);
    int red     = gdImageColorAllocate(im_out,255,  0,  0);
    int yellow  = gdImageColorAllocate(im_out,240,255,  0);
    int navy    = gdImageColorAllocate(im_out,  0,  0,128);
    int gray    = gdImageColorAllocate(im_out,128,128,128);
    int dgray   = gdImageColorAllocate(im_out, 64, 64, 64);
    int black   = gdImageColorAllocate(im_out,  0,  0,  0);
    int dgreen  = gdImageColorAllocate(im_out, 0,255,  0);
    int fuchsia = gdImageColorAllocate(im_out, 255, 0, 255);
    int teal    = gdImageColorAllocate(im_out, 5, 237, 255);
    //gdImageColorTransparent(im_out,white);

    // Ausserer Rahmen (Bildrand gefllt mit grau)
    gdImageFilledRectangle(im_out,ALeft,ATop,width,height,gray);

    // Innerer Rahmen (Achsen- und Labelflaeche gefllt mit weiss)
    gdImageFilledRectangle(im_out,5,5,width-5,height-5,white);

    // Plottflche in navy ... black
    gdImageFilledRectangle(im_out,BLeft,BTop,BRight,BBottom,navy);

    // y-ticks and y-labels
    char Label[40];
    int ylabelcolor = black;
    int Ny = 7;  // number of zwischenrume
       YMin = ((int)(YMin/1))*1;
       YMax = (((int)(YMax/1))+1)*1;
       if(!calibrated)
       {
         YMin = 0;
         YMax = Form1->yscale; // 20 dB
         Ny = 10;
       }
    float ytick = (YMax-YMin)/Ny; // y-increment
    float yrange = float(Ny)*ytick;
    float dy = (float)(BBottom-BTop)/(float)Ny;
    int i = 0;
    int x1 = BLeft; int x2 = x1-5;
    float y;
    int w, h;
    do {
        y = (float)BTop + (float)i*dy;
        gdImageLine(im_out,x1,y,x2,y,ylabelcolor);
        sprintf(Label,"%7.1f",((Ny-i)*ytick)+YMin); // y-label
        w = 7*6; // Schtzung von gdFontSmall
        h = 12;  // Schtzung von gdFontSmall
        gdImageString(im_out,gdFontSmall,BLeft-6-w,y-h/2-2,(unsigned char *)Label,ylabelcolor);
        i++;
    } while (i<=Ny);
    sprintf(Label,"%s",ylabel); // y-label
    gdImageString(im_out,gdFontSmall,ALeft+8,BTop-8-h,(unsigned char *)Label,ylabelcolor);

    // x-ticks and x-labels
    int Nx = 24;
    int xlabelcolor = black;
    float dx = (float)(BRight-BLeft)/(float)Nx;
    i = 0;
    int y1 = BBottom; int y2 = y1+5; int y3 = y2+4;  // short (y2) and long (y3) ticks
    float x;
    do {
        x = (float)BLeft+(float)i*dx;
        gdImageLine(im_out,x,y1,x,y2,xlabelcolor);
        sprintf(Label,"%02d",i); // x-label
        w = 2*6; // Schtzung von gdFontSmall
        h = 12;  // Schtzung von gdFontSmall
        if ((i%2)==0)
        {
            gdImageLine(im_out,x,y1,x,y3,xlabelcolor);
            gdImageString(im_out,gdFontSmall,x-w/2+1,y2+h/2,(unsigned char *)Label,xlabelcolor);
        }
        i++;
    } while (i<=Nx);
    sprintf(Label,"Observation Time [UT] of %02i/%02i/%04i",Day,Month,Year); // y-label
    w = (22+10)*6; // Schtzung von gdFontSmall
    h = 12;  // Schtzung von gdFontSmall
    gdImageString(im_out,gdFontSmall,(BLeft+BRight)/2-w/2,y3+h,(unsigned char *)Label,xlabelcolor);

    // title
    int titlecolor=dgray;
    sprintf(Label,"Lightcurves Callisto %s",Form1->titlecomment); // title
    w = 40*8; // Schtzung von gdFontGiant
    w = 40*6; // Schtzung von gdFontSmall
    w = (strlen(Label)+3)*7; // Schtzung von gdFontLarge
    h = 16;   // Schtzung von gdFontGiant
    gdImageString(im_out,gdFontLarge,(ALeft+ARight)/2-w/2,BTop-6-h,(unsigned char *)Label,titlecolor);

//LC1-------------------------------------------------------------------------------------------------
    int y0 = BBottom-2;
    int x0 = BLeft;
    float gain = (float)(BBottom-BTop)/yrange;  // height/scale
    int Nt = counts-1; // Anzahl Messwerte
    float px;
    float py;
    static float px_old = x0, py_old = y0;
    int plottcolor = yellow;
    float tact, yact;
    char st[80];
    int lx = BRight -55; // position label xxxMHz
    gdImageLine(im_out,x0,y0,x0,y0,navy);
    if (counts>0)
    {
      for (int t=1; t<Nt; t++)
      {
          tact =  LightCurvesBicone[t].MEZ;
          if (calibrated)  yact =  LightCurvesBicone[t].LC1;
          else yact =  (LightCurvesBicone[t].LC1-IMin_a)/((256.0*Form1->adcdivider)-1)*2500.0/grad; // =ImV/25.4mV/dB
          if (tact <  0) tact =  0;
          if (tact > 24) tact = 24;
          px = x0 + tact*(float)(BRight-BLeft)/24.0;
          py = y0 - gain*(yact-YMin);
          if (t<2) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel
          if (px > px_old && t>1) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel //gdImageLine(im_out,px_old,py_old,px,py,plottcolor); // line
          sprintf(value,"%3.1f MHz",nameLC1);
          gdImageString(im_out,gdFontSmall,lx, 35,(unsigned char*)value,plottcolor);
          px_old = px;
          py_old = py;
      }
    }
//LC2-------------------------------------------------------------------------------------------------
    y0 = BBottom-2;
    x0 = BLeft;
    gain = (float)(BBottom-BTop)/yrange;  // height/scale
    Nt = counts-1; // Anzahl Messwerte
    px_old = x0, py_old = y0;
    plottcolor = red;
    if (counts>0)
    {
      for (int t=1; t<Nt; t++)
      {
          tact =  LightCurvesBicone[t].MEZ;
          if (calibrated)  yact =  LightCurvesBicone[t].LC2;
          else yact =  (LightCurvesBicone[t].LC2-IMin_b)/((256.0*Form1->adcdivider)-1)*2500.0/grad; // =ImV/25.4mV/dB
          if (tact <  0) tact =  0;
          if (tact > 24) tact = 24;
          px = x0 + tact*(float)(BRight-BLeft)/24.0;
          py = y0 - gain*(yact-YMin);
          if (t<2) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel
          if (px > px_old && t>1) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel //gdImageLine(im_out,px_old,py_old,px,py,plottcolor); // line
          sprintf(value,"%3.1f MHz",nameLC2);
          gdImageString(im_out,gdFontSmall,lx, 45,(unsigned char*)value,plottcolor);
          px_old = px;
          py_old = py;
       }
    }
//LC3-------------------------------------------------------------------------------------------------
    y0 = BBottom-2;
    x0 = BLeft;
    gain = (float)(BBottom-BTop)/yrange;  // height/scale
    Nt = counts-1; // Anzahl Messwerte
    px_old = x0, py_old = y0;
    plottcolor = dgreen;
    if (counts>0)
    {
      for (int t=1; t<Nt; t++)
      {
          tact =  LightCurvesBicone[t].MEZ;
          if (calibrated)  yact =  LightCurvesBicone[t].LC3;
          else yact =  (LightCurvesBicone[t].LC3-IMin_c)/((256.0*Form1->adcdivider)-1)*2500.0/grad; // =ImV/25.4mV/dB
          if (tact <  0) tact =  0;
          if (tact > 24) tact = 24;
          px = x0 + tact*(float)(BRight-BLeft)/24.0;
          py = y0 - gain*(yact-YMin);
          if (t<2) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel
          if (px > px_old && t>1) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel //gdImageLine(im_out,px_old,py_old,px,py,plottcolor); // line
          sprintf(value,"%3.1f MHz",nameLC3);
          gdImageString(im_out,gdFontSmall,lx, 55,(unsigned char*)value,plottcolor);
          px_old = px;
          py_old = py;
      }
    }
//LC4-------------------------------------------------------------------------------------------------
    y0 = BBottom-2;
    x0 = BLeft;
    gain = (float)(BBottom-BTop)/yrange;  // height/scale
    Nt = counts-1; // Anzahl Messwerte
    px_old = x0, py_old = y0;
    plottcolor = fuchsia;
    if (counts>0)
    {
      for (int t=1; t<Nt; t++)
      {
          tact =  LightCurvesBicone[t].MEZ;
          if (calibrated)  yact =  LightCurvesBicone[t].LC3;
          else yact =  (LightCurvesBicone[t].LC4-IMin_d)/((256.0*Form1->adcdivider)-1)*2500.0/grad; // =ImV/25.4mV/dB
          if (tact <  0) tact =  0;
          if (tact > 24) tact = 24;
          px = x0 + tact*(float)(BRight-BLeft)/24.0;
          py = y0 - gain*(yact-YMin);
          if (t<2) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel
          if (px > px_old && t>1) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel //gdImageLine(im_out,px_old,py_old,px,py,plottcolor); // line
          sprintf(value,"%3.1f MHz",nameLC4);
          gdImageString(im_out,gdFontSmall,lx, 65,(unsigned char*)value,plottcolor);
          px_old = px;
          py_old = py;
      }
    }

//LC5-------------------------------------------------------------------------------------------------
    y0 = BBottom-2;
    x0 = BLeft;
    gain = (float)(BBottom-BTop)/yrange;  // height/scale
    Nt = counts-1; // Anzahl Messwerte
    px_old = x0, py_old = y0;
    plottcolor = white;
    if (counts>0)
    {
      for (int t=1; t<Nt; t++)
      {
          tact =  LightCurvesBicone[t].MEZ;
          if (calibrated)  yact =  LightCurvesBicone[t].LC5;
          else yact =  (LightCurvesBicone[t].LC5-IMin_e)/((256.0*Form1->adcdivider)-1)*2500.0/grad; // =ImV/25.4mV/dB
          if (tact <  0) tact =  0;
          if (tact > 24) tact = 24;
          px = x0 + tact*(float)(BRight-BLeft)/24.0;
          py = y0 - gain*(yact-YMin);
          if (t<2) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel
          if (px > px_old && t>1) gdImageLine(im_out,px,py,px,py,plottcolor); // pixel //gdImageLine(im_out,px_old,py_old,px,py,plottcolor); // line
          sprintf(value,"%3.1f MHz",nameLC5);
          gdImageString(im_out,gdFontSmall,lx, 75,(unsigned char*)value,plottcolor);
          px_old = px;
          py_old = py;
      }
    }


    // save image
    char imagefile[240];
    if (archivPng)
    {
         sprintf(imagefile,"%sLightcurves%s_%04u%02u%02u.png",Form1->imagedir,Form1->titlecomment,Year,Month,Day);
         archivPng = false;

    }
    else
    {
       sprintf(imagefile,"%sLightcurves%s.png",Form1->imagedir,Form1->titlecomment);
    }
    FILE *out;
    out = fopen(imagefile,"wb");
    gdImagePng(im_out,out); // save
    fclose(out);
    gdImageDestroy(im_out);

    Form1->Label5->Caption = imagefile;
}
//---------------------------------------------------------------------------
