/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: xyplott.cpp                                            */
/*                                                                       */
/*  Revision: V1.0     Date: 22.04.2009    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Generiert aus ASCII-Datei einen XY-Plott, z.B. Windgeschw.  */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 22.04.2009
// Updated by: A. James 23.06.2009
// Updated by: Chr. Monstein 04.08.2010 (Phoenix-3 again)
// Updated by: Chr. Monstein 02.09.2010 adaption to Callisto Metsahovi
// Updated by: Chr. Monstein 04.04.2011 more general, reads callsito.cfg
// Updated by: Chr. Monstein 14.08.2011 width, height = parameter
// Updated by: Chr. Monstein 08.09.2011 dB korrigiert
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "xyplott.h"
#include <stdio.h>
#include <math.h>
#include <fstream.h> // ifstream
#include "config.h"
#include "LCBicone.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
CLCBicone LCBicone;
char configfile[200]; // location of callisto.cfg

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
  ReadConfig(); // own, local config
  ReadConfigCallisto(); // get callisto.cfg and LCpath

  first_pass = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ReadConfig(void)
{
  char buff[240];
  AnsiString st;

  Timer2->Enabled = false;
  ifstream infile("wwwpar.cfg"); // must be local
  while (infile)
  {
      infile.getline(buff,240);
      st = LowerCase(AnsiString(buff));
      if (buff[0]=='[')
      {
        if (strstr(st.c_str(),"[configfile]="))      sscanf(buff,"[configfile]=%s"      ,&configfile);
        if (strstr(st.c_str(),"[imagedir]="))        sscanf(buff,"[imagedir]=%s"        ,&imagedir);
        if (strstr(st.c_str(),"[timer]="))           sscanf(buff,"[timer]=%i"           ,&timerinterval);
        if (strstr(st.c_str(),"[dayoffset]="))       sscanf(buff,"[dayoffset]=%u"       ,&dayoffset);
        if (strstr(st.c_str(),"[yscale]="))          sscanf(buff,"[yscale]=%i"          ,&yscale);
        if (strstr(st.c_str(),"[width]="))           sscanf(buff,"[width]=%i"           ,&p_width);
        if (strstr(st.c_str(),"[height]="))          sscanf(buff,"[height]=%i"          ,&p_height);
        if (strstr(st.c_str(),"[adcdivider]="))      sscanf(buff,"[adcdivider]=%i"      ,&adcdivider);
      }
  }
  infile.close();

  //if (!DirectoryExists(AnsiString(configfile)))
  if (!FileExists(AnsiString(configfile)))
  {
    configfile[0]='\0';  // set to local
    Application->MessageBox("Path does not exist", "Callisto configfile path error", MB_OK);
  }

  if (!DirectoryExists(AnsiString(imagedir)))
  {
    imagedir[0]='\0';  // set to local
    Application->MessageBox("Path does not exist", "Image directory path error", MB_OK);
  }
  if (timerinterval !=0) Timer2->Interval = timerinterval*1000;  // sec -> msec
  if (yscale < 1) yscale = 20;
  Timer2->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ReadConfigCallisto(void)
{
  char buff[240];
  AnsiString st;

  ifstream infile(configfile);
  while (infile)
  {
      infile.getline(buff,240);
      st = LowerCase(AnsiString(buff));
      if (buff[0]=='[')
      {
        if (strstr(st.c_str(),"[titlecomment]="))    sscanf(buff,"[titlecomment]=%s"   ,&titlecomment);
        if (strstr(st.c_str(),"[lcpath]="))          sscanf(buff,"[lcpath]=%s"         ,&LCpath);

      }
  }
  infile.close();

  if (!DirectoryExists(AnsiString(LCpath)))
  {
    LCpath[0]='\0';  // set to local
    Application->MessageBox("Path does not exist", "Light curve path error", MB_OK);
  }
  Form1->Caption = "PNG plotter Callisto " + AnsiString(titlecomment);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer2Timer(TObject *Sender)
{
  Word Hour, Min, Sec, MSec, Year, Month, Day;
  static int state = 0;
  switch(state++)
  {
    case 0:
    {
       Label1->Caption = DateTimeToStr(Now());
       Label2->Caption = "Generate lightcurves Callisto";
       LCBicone.Read();
       LCBicone.GdImageGenerator();
       break;
    }
    case 1:
    {
       Label1->Caption = DateTimeToStr(Now());
       Label2->Caption = "Wait....";
       first_pass = false;
       state = 0;
       break;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
   Timer2->Enabled=false;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::CheckBox1Click(TObject *Sender)
{
  if(CheckBox1->Checked)
    Timer2->Interval=500;
  else
    Timer2->Interval=timerinterval*1000;
}
//---------------------------------------------------------------------------

